-- ==============================================================
-- FILE : HE_DDR_DATA.VHD  -  DO NOT EDIT THIS FILE
-- ==============================================================
--
-- This file is part of the Hardware Interface Layer (H.I.L) of
-- your project when using your HERON-FPGA9 module.
--
-- This file must NOT be modified by users of the Module.
--
-- ==============================================================
--
--  Module      : HE_DDR_DATA
--  Date        : 01/07/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : DDR SDRAM Interface Data Path
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    01-07-04    First Written
--  1.1     R. Williams    01-08-05    Changed use of 'rdrise_ef'
--                                     to 'rdrise_af' and changed
--                                     'rdfall_ef' to 'rdfall_af'
--                                     for read address fifo read
--                                     enable logic. This was done
--                                     to improve DDR read speeds.
--  2.0     R. Williams    16-09-05    Added PowerPC to DDR memory
--                                     interface.
--  2.1     R. Williams    09-06-06    Changed RD/WR select logic
--                                     for the PowerPC to ensure
--                                     processor accesses occur
--                                     in the correct order.
--  2.2     R. Williams    28-08-07    Added more IOB attributes
--                                     to ensure correct DDR IOB
--                                     instanciation in ISE9.1
--  2.3     R. Williams    27-09-07    Change made to control
--                                     logic to ensure write has
--                                     priority over read while
--                                     both ready in the same row.
--  2.4     R. Williams    11-03-08    Adjusted DDR read timing
--                                     due to Place and Route
--                                     changes in ISE9.2.
--  2.5     R. Williams    29-04-09    Changed logic controlling
--                                     the DDR read-data input FF
--                                     clock enables to improve
--                                     DDR data margins.
--
-- ==============================================================



-- ==============================================================
--  Module      : HE_DDR_A_DATA
--  Description : DDR Port A Data

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
  use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_A_DATA is
  port(
    user_input_data   : in  std_logic_vector(71 downto 0);
    clk0              : in  std_logic;
    clk180            : in  std_logic;
    reset             : in  std_logic;
    DQS_enable        : in  std_logic;
    DQS_reset         : in  std_logic;
    write_enable      : in  std_logic;
    read_enable       : in  std_logic;
    read_select       : in  std_logic;
    wclk              : in  std_logic;
    ddr_dq_in         : in  std_logic_vector(31 downto 0);
    ddr_dq_out        : out std_logic_vector(31 downto 0);
    ddr_dq_en         : out std_logic_vector(31 downto 0);
    ddr_dqs_in        : in  std_logic_vector(3 downto 0);
    ddr_dqs_out       : out std_logic_vector(3 downto 0);
    ddr_dqs_en        : out std_logic_vector(3 downto 0);
    ddr_dm_out        : out std_logic_vector(3 downto 0);
    ddr_dm_en         : out std_logic_vector(3 downto 0);
    read_data_rising  : out std_logic_vector(31 downto 0);
    read_data_falling : out std_logic_vector(31 downto 0);
    control_a1        : out std_logic;
    control_a2        : out std_logic;
    control_ab1       : out std_logic;
    control_ab2       : out std_logic
  );
end HE_DDR_A_DATA;

architecture RTL of HE_DDR_A_DATA is

  component FDC
    port (
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      CLR : in  std_logic
    );
  end component;

  component HE_DDR_A_IOB_A
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_A_IOB_B
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_A_IOB_C
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_A_IOB_D
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_DQS
    port (
      DQS_enable  : in  std_logic;
      DQS_reset   : in  std_logic;
      clk         : in  std_logic;
      clk180      : in  std_logic;
      DDR_DQS_IN  : in  std_logic_vector(3 downto 0);
      DDR_DQS_OUT : out std_logic_vector(3 downto 0);
      DDR_DQS_EN  : out std_logic_vector(3 downto 0);
      DQS         : out std_logic_vector(3 downto 0)
    );
  end component;

  component HE_DDR_DM_IOB
    port (
      write_data_mask_rising  : in  std_logic;
      write_data_mask_falling : in  std_logic;
      wclk                    : in  std_logic;
      reset                   : in  std_logic;
      write_en_val            : in  std_logic;
      ddr_dm_out              : out std_logic;
      ddr_dm_en               : out std_logic
    );
  end component;

  attribute LOC : string;

  attribute LOC of iRD_EN_A : label is "SLICE_X66Y49";
  attribute LOC of iRD_EN_B : label is "SLICE_X66Y57";
  attribute LOC of iRD_EN_C : label is "SLICE_X66Y65";
  attribute LOC of iRD_EN_D : label is "SLICE_X66Y73";

  signal DELAY_SELECT            : std_logic_vector(7 downto 0);

  signal dqs                     : std_logic_vector(3 downto 0);

  signal write_data_rising       : std_logic_vector(31 downto 0);
  signal write_data_falling      : std_logic_vector(31 downto 0);
  signal write_data_mask_rising  : std_logic_vector(3 downto 0);
  signal write_data_mask_falling : std_logic_vector(3 downto 0);
  signal write_en_val            : std_logic;
  signal read_en_P4              : std_logic;
  signal read_en_P5              : std_logic;

  signal read_sel_P4             : std_logic;
  signal read_sel_P5             : std_logic;

  signal read_en_val_a           : std_logic;
  signal read_en_val_b           : std_logic;
  signal read_en_val_c           : std_logic;
  signal read_en_val_d           : std_logic;

  signal read_en_val             : std_logic;
  signal read_en_valb            : std_logic;

  signal read_sel_val            : std_logic;
  signal read_sel_valb           : std_logic;

begin

  -- Delay Value for 200MHz Operation on Bank A (-6 Speed Grade)
  DELAY_SELECT <= "11110111";

  -- Delay Value for 167MHz Operation on Bank A (-5 Speed Grade)
  --DELAY_SELECT <= "11110111";

  process(reset, wclk)
  begin
    if reset='1' then
      write_data_rising  <= (others=>'0');
      write_data_falling <= (others=>'0');
    elsif wclk'event and wclk='0' then
      write_data_rising  <= user_input_data(39 downto 8);
      write_data_falling <= user_input_data(71 downto 40);
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      write_data_mask_rising  <= (others=>'0');
      write_data_mask_falling <= (others=>'0');
    elsif wclk'event and wclk='0' then
      write_data_mask_rising  <= user_input_data(7 downto 4);
      write_data_mask_falling <= user_input_data(3 downto 0);
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      write_en_val <= '0';
    elsif wclk'event and wclk='0' then
      write_en_val <= write_enable;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      read_en_P4  <= '0';
      read_sel_P4 <= '0';
    elsif wclk'event and wclk='0' then
      read_en_P4  <= read_enable;
      read_sel_P4 <= read_select;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      read_en_P5   <= '0';
      read_sel_P5  <= '0';
      read_en_val  <= '0';
      read_sel_val <= '0';
    elsif wclk'event and wclk='1' then
      read_en_P5   <= read_en_P4;
      read_sel_P5  <= read_sel_P4;
      read_en_val  <= read_en_P5;
      read_sel_val <= read_sel_P5;
    end if;
  end process;

  iRD_EN_A : FDC port map (Q=>read_en_val_a, D=>read_en_P5, C=>wclk, CLR=>reset);
  iRD_EN_B : FDC port map (Q=>read_en_val_b, D=>read_en_P5, C=>wclk, CLR=>reset);
  iRD_EN_C : FDC port map (Q=>read_en_val_c, D=>read_en_P5, C=>wclk, CLR=>reset);
  iRD_EN_D : FDC port map (Q=>read_en_val_d, D=>read_en_P5, C=>wclk, CLR=>reset);

  process(reset, wclk)
  begin
    if reset='1' then
      read_en_valb  <= '0';
      read_sel_valb <= '0';
    elsif wclk'event and wclk='0' then
      read_en_valb  <= read_en_val;
      read_sel_valb <= read_sel_val;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      control_a1 <= '0';
      control_a2 <= '0';
    elsif wclk'event and wclk='1' then
      control_a1 <= '0';
      control_a2 <= '0';
      if (read_en_val='1' and read_sel_val='0') then
        control_a1 <= '1';
      end if;
      if (read_en_val='1' and read_sel_val='1') then
        control_a2 <= '1';
      end if;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      control_ab1 <= '0';
      control_ab2 <= '0';
    elsif wclk'event and wclk='0' then
      control_ab1 <= '0';
      control_ab2 <= '0';
      if (read_en_valb='1' and read_sel_valb='0') then
        control_ab1 <= '1';
      end if;
      if (read_en_valb='1' and read_sel_valb='1') then
        control_ab2 <= '1';
      end if;
    end if;
  end process;

  U_BYTE0 : HE_DDR_A_IOB_A
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(0),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_a,
      write_data_rising  => write_data_rising(7 downto 0),
      write_data_falling => write_data_falling(7 downto 0),
      read_data_rising   => read_data_rising(7 downto 0),
      read_data_falling  => read_data_falling(7 downto 0),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(7 downto 0),
      ddr_data_out       => ddr_dq_out(7 downto 0),
      ddr_data_en        => ddr_dq_en(7 downto 0) );

  U_BYTE1 : HE_DDR_A_IOB_B
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(1),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_b,
      write_data_rising  => write_data_rising(15 downto 8),
      write_data_falling => write_data_falling(15 downto 8),
      read_data_rising   => read_data_rising(15 downto 8),
      read_data_falling  => read_data_falling(15 downto 8),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(15 downto 8),
      ddr_data_out       => ddr_dq_out(15 downto 8),
      ddr_data_en        => ddr_dq_en(15 downto 8) );

  U_BYTE2 : HE_DDR_A_IOB_C
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(2),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_c,
      write_data_rising  => write_data_rising(23 downto 16),
      write_data_falling => write_data_falling(23 downto 16),
      read_data_rising   => read_data_rising(23 downto 16),
      read_data_falling  => read_data_falling(23 downto 16),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(23 downto 16),
      ddr_data_out       => ddr_dq_out(23 downto 16),
      ddr_data_en        => ddr_dq_en(23 downto 16) );

  U_BYTE3 : HE_DDR_A_IOB_D
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(3),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_d,
      write_data_rising  => write_data_rising(31 downto 24),
      write_data_falling => write_data_falling(31 downto 24),
      read_data_rising   => read_data_rising(31 downto 24),
      read_data_falling  => read_data_falling(31 downto 24),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(31 downto 24),
      ddr_data_out       => ddr_dq_out(31 downto 24),
      ddr_data_en        => ddr_dq_en(31 downto 24) );

  INST_2 : HE_DDR_DQS
    port map (
      DQS_enable  => DQS_enable,
      DQS_reset   => DQS_reset,
      clk         => clk0,
      clk180      => clk180,
      DDR_DQS_IN  => ddr_dqs_in,
      DDR_DQS_OUT => ddr_dqs_out,
      DDR_DQS_EN  => ddr_dqs_en,
      DQS         => dqs );

  U_DMASK_byte0 : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(0),
      write_data_mask_falling => write_data_mask_falling(0),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(0),
      ddr_dm_en               => ddr_dm_en(0) );

  U_DMASK_byte1 : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(1),
      write_data_mask_falling => write_data_mask_falling(1),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(1),
      ddr_dm_en               => ddr_dm_en(1) );

  U_DMASK_byte2 : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(2),
      write_data_mask_falling => write_data_mask_falling(2),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(2),
      ddr_dm_en               => ddr_dm_en(2) );

  U_DMASK_byte : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(3),
      write_data_mask_falling => write_data_mask_falling(3),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(3),
      ddr_dm_en               => ddr_dm_en(3) );

end RTL;


-- ==============================================================
--  Module      : HE_DDR_B_DATA
--  Description : DDR Port B Data

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
  use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_B_DATA is
  port(
    user_input_data   : in  std_logic_vector(71 downto 0);
    clk0              : in  std_logic;
    clk180            : in  std_logic;
    reset             : in  std_logic;
    DQS_enable        : in  std_logic;
    DQS_reset         : in  std_logic;
    write_enable      : in  std_logic;
    read_enable       : in  std_logic;
    read_select       : in  std_logic;
    wclk              : in  std_logic;
    ddr_dq_in         : in  std_logic_vector(31 downto 0);
    ddr_dq_out        : out std_logic_vector(31 downto 0);
    ddr_dq_en         : out std_logic_vector(31 downto 0);
    ddr_dqs_in        : in  std_logic_vector(3 downto 0);
    ddr_dqs_out       : out std_logic_vector(3 downto 0);
    ddr_dqs_en        : out std_logic_vector(3 downto 0);
    ddr_dm_out        : out std_logic_vector(3 downto 0);
    ddr_dm_en         : out std_logic_vector(3 downto 0);
    read_data_rising  : out std_logic_vector(31 downto 0);
    read_data_falling : out std_logic_vector(31 downto 0);
    control_a1        : out std_logic;
    control_a2        : out std_logic;
    control_ab1       : out std_logic;
    control_ab2       : out std_logic
  );
end HE_DDR_B_DATA;

architecture RTL of HE_DDR_B_DATA is

  component FDC
    port (
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic;
      CLR : in  std_logic
    );
  end component;

  component HE_DDR_B_IOB_A
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_B_IOB_B
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_B_IOB_C
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_B_IOB_D
    port (
      reset              : in  std_logic;
      wclk               : in  std_logic;
      dqs                : in  std_logic;
      write_en_val       : in  std_logic;
      read_en_val        : in  std_logic;
      write_data_rising  : in  std_logic_vector(7 downto 0);
      write_data_falling : in  std_logic_vector(7 downto 0);
      read_data_rising   : out std_logic_vector(7 downto 0);
      read_data_falling  : out std_logic_vector(7 downto 0);
      DELAY_SELECT       : in  std_logic_vector(7 downto 0);
      ddr_data_in        : in  std_logic_vector(7 downto 0);
      ddr_data_out       : out std_logic_vector(7 downto 0);
      ddr_data_en        : out std_logic_vector(7 downto 0)
    );
  end component;

  component HE_DDR_DQS
    port (
      DQS_enable  : in  std_logic;
      DQS_reset   : in  std_logic;
      clk         : in  std_logic;
      clk180      : in  std_logic;
      DDR_DQS_IN  : in  std_logic_vector(3 downto 0);
      DDR_DQS_OUT : out std_logic_vector(3 downto 0);
      DDR_DQS_EN  : out std_logic_vector(3 downto 0);
      DQS         : out std_logic_vector(3 downto 0)
    );
  end component;

  component HE_DDR_DM_IOB
    port (
      write_data_mask_rising  : in  std_logic;
      write_data_mask_falling : in  std_logic;
      wclk                    : in  std_logic;
      reset                   : in  std_logic;
      write_en_val            : in  std_logic;
      ddr_dm_out              : out std_logic;
      ddr_dm_en               : out std_logic
    );
  end component;

  attribute LOC : string;

  attribute LOC of iRD_EN_A : label is "SLICE_X66Y9";
  attribute LOC of iRD_EN_B : label is "SLICE_X66Y17";
  attribute LOC of iRD_EN_C : label is "SLICE_X66Y25";
  attribute LOC of iRD_EN_D : label is "SLICE_X66Y33";

  signal DELAY_SELECT            : std_logic_vector(7 downto 0);

  signal dqs                     : std_logic_vector(3 downto 0);

  signal write_data_rising       : std_logic_vector(31 downto 0);
  signal write_data_falling      : std_logic_vector(31 downto 0);
  signal write_data_mask_rising  : std_logic_vector(3 downto 0);
  signal write_data_mask_falling : std_logic_vector(3 downto 0);
  signal write_en_val            : std_logic;
  signal read_en_P4              : std_logic;
  signal read_en_P5              : std_logic;

  signal read_sel_P4             : std_logic;
  signal read_sel_P5             : std_logic;

  signal read_en_val_a           : std_logic;
  signal read_en_val_b           : std_logic;
  signal read_en_val_c           : std_logic;
  signal read_en_val_d           : std_logic;

  signal read_en_val             : std_logic;
  signal read_en_valb            : std_logic;

  signal read_sel_val            : std_logic;
  signal read_sel_valb           : std_logic;

begin

  -- Delay Value for 200MHz Operation on Bank B (-6 Speed Grade)
  DELAY_SELECT <= "11110111";

  -- Delay Value for 167MHz Operation on Bank B (-5 Speed Grade)
  --DELAY_SELECT <= "11110111";

  process(reset, wclk)
  begin
    if reset='1' then
      write_data_rising  <= (others=>'0');
      write_data_falling <= (others=>'0');
    elsif wclk'event and wclk='0' then
      write_data_rising  <= user_input_data(39 downto 8);
      write_data_falling <= user_input_data(71 downto 40);
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      write_data_mask_rising  <= (others=>'0');
      write_data_mask_falling <= (others=>'0');
    elsif wclk'event and wclk='0' then
      write_data_mask_rising  <= user_input_data(7 downto 4);
      write_data_mask_falling <= user_input_data(3 downto 0);
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      write_en_val <= '0';
    elsif wclk'event and wclk='0' then
      write_en_val <= write_enable;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      read_en_P4  <= '0';
      read_sel_P4 <= '0';
    elsif wclk'event and wclk='0' then
      read_en_P4  <= read_enable;
      read_sel_P4 <= read_select;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      read_en_P5   <= '0';
      read_sel_P5  <= '0';
      read_en_val  <= '0';
      read_sel_val <= '0';
    elsif wclk'event and wclk='1' then
      read_en_P5   <= read_en_P4;
      read_sel_P5  <= read_sel_P4;
      read_en_val  <= read_en_P5;
      read_sel_val <= read_sel_P5;
    end if;
  end process;

  iRD_EN_A : FDC port map (Q=>read_en_val_a, D=>read_en_P5, C=>wclk, CLR=>reset);
  iRD_EN_B : FDC port map (Q=>read_en_val_b, D=>read_en_P5, C=>wclk, CLR=>reset);
  iRD_EN_C : FDC port map (Q=>read_en_val_c, D=>read_en_P5, C=>wclk, CLR=>reset);
  iRD_EN_D : FDC port map (Q=>read_en_val_d, D=>read_en_P5, C=>wclk, CLR=>reset);

  process(reset, wclk)
  begin
    if reset='1' then
      read_en_valb  <= '0';
      read_sel_valb <= '0';
    elsif wclk'event and wclk='0' then
      read_en_valb  <= read_en_val;
      read_sel_valb <= read_sel_val;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      control_a1 <= '0';
      control_a2 <= '0';
    elsif wclk'event and wclk='1' then
      control_a1 <= '0';
      control_a2 <= '0';
      if (read_en_val='1' and read_sel_val='0') then
        control_a1 <= '1';
      end if;
      if (read_en_val='1' and read_sel_val='1') then
        control_a2 <= '1';
      end if;
    end if;
  end process;

  process(reset, wclk)
  begin
    if reset='1' then
      control_ab1 <= '0';
      control_ab2 <= '0';
    elsif wclk'event and wclk='0' then
      control_ab1 <= '0';
      control_ab2 <= '0';
      if (read_en_valb='1' and read_sel_valb='0') then
        control_ab1 <= '1';
      end if;
      if (read_en_valb='1' and read_sel_valb='1') then
        control_ab2 <= '1';
      end if;
    end if;
  end process;

  U_BYTE0 : HE_DDR_B_IOB_A
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(0),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_a,
      write_data_rising  => write_data_rising(7 downto 0),
      write_data_falling => write_data_falling(7 downto 0),
      read_data_rising   => read_data_rising(7 downto 0),
      read_data_falling  => read_data_falling(7 downto 0),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(7 downto 0),
      ddr_data_out       => ddr_dq_out(7 downto 0),
      ddr_data_en        => ddr_dq_en(7 downto 0) );

  U_BYTE1 : HE_DDR_B_IOB_B
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(1),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_b,
      write_data_rising  => write_data_rising(15 downto 8),
      write_data_falling => write_data_falling(15 downto 8),
      read_data_rising   => read_data_rising(15 downto 8),
      read_data_falling  => read_data_falling(15 downto 8),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(15 downto 8),
      ddr_data_out       => ddr_dq_out(15 downto 8),
      ddr_data_en        => ddr_dq_en(15 downto 8) );

  U_BYTE2 : HE_DDR_B_IOB_C
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(2),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_c,
      write_data_rising  => write_data_rising(23 downto 16),
      write_data_falling => write_data_falling(23 downto 16),
      read_data_rising   => read_data_rising(23 downto 16),
      read_data_falling  => read_data_falling(23 downto 16),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(23 downto 16),
      ddr_data_out       => ddr_dq_out(23 downto 16),
      ddr_data_en        => ddr_dq_en(23 downto 16) );

  U_BYTE3 : HE_DDR_B_IOB_D
    port map (
      reset              => reset,
      wclk               => wclk,
      dqs                => dqs(3),
      write_en_val       => write_en_val,
      read_en_val        => read_en_val_d,
      write_data_rising  => write_data_rising(31 downto 24),
      write_data_falling => write_data_falling(31 downto 24),
      read_data_rising   => read_data_rising(31 downto 24),
      read_data_falling  => read_data_falling(31 downto 24),
      DELAY_SELECT       => DELAY_SELECT,
      ddr_data_in        => ddr_dq_in(31 downto 24),
      ddr_data_out       => ddr_dq_out(31 downto 24),
      ddr_data_en        => ddr_dq_en(31 downto 24) );

  INST_2 : HE_DDR_DQS
    port map (
      DQS_enable  => DQS_enable,
      DQS_reset   => DQS_reset,
      clk         => clk0,
      clk180      => clk180,
      DDR_DQS_IN  => ddr_dqs_in,
      DDR_DQS_OUT => ddr_dqs_out,
      DDR_DQS_EN  => ddr_dqs_en,
      DQS         => dqs );

  U_DMASK_byte0 : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(0),
      write_data_mask_falling => write_data_mask_falling(0),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(0),
      ddr_dm_en               => ddr_dm_en(0) );

  U_DMASK_byte1 : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(1),
      write_data_mask_falling => write_data_mask_falling(1),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(1),
      ddr_dm_en               => ddr_dm_en(1) );

  U_DMASK_byte2 : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(2),
      write_data_mask_falling => write_data_mask_falling(2),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(2),
      ddr_dm_en               => ddr_dm_en(2) );

  U_DMASK_byte : HE_DDR_DM_IOB
    port map (
      write_data_mask_rising  => write_data_mask_rising(3),
      write_data_mask_falling => write_data_mask_falling(3),
      wclk                    => wclk,
      reset                   => reset,
      write_en_val            => write_en_val,
      ddr_dm_out              => ddr_dm_out(3),
      ddr_dm_en               => ddr_dm_en(3) );

end RTL;


-- ==============================================================
--  Module      : HE_DDR_A_IOB_A
--  Description : DDR Port A Data, Byte 0

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_A_IOB_A is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_A_IOB_A;


architecture RTL of HE_DDR_A_IOB_A is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y47";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y46";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y47";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y46";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y46";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y44";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y45";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y44";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y45";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y44";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y45";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y44";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y45";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y48";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y49";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y48";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y49";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y50";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y51";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y50";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y51";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y48";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;
  signal clk_delay_6 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_A_IOB_B
--  Description : DDR Port A Data, Byte 1

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_A_IOB_B is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_A_IOB_B;


architecture RTL of HE_DDR_A_IOB_B is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y55";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y54";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y55";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y54";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y54";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y52";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y53";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y52";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y53";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y52";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y53";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y52";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y53";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y56";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y57";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y56";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y57";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y58";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y59";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y58";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y59";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y56";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;
  signal clk_delay_6 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_A_IOB_C
--  Description : DDR Port A Data, Byte 2

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_A_IOB_C is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_A_IOB_C;


architecture RTL of HE_DDR_A_IOB_C is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y63";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y62";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y63";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y62";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y62";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y60";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y61";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y60";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y61";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y60";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y61";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y60";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y61";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y64";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y65";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y64";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y65";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y66";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y67";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y66";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y67";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y64";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;
  signal clk_delay_6 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_A_IOB_D
--  Description : DDR Port A Data, Byte 3

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_A_IOB_D is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_A_IOB_D;


architecture RTL of HE_DDR_A_IOB_D is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y71";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y70";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y71";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y70";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y70";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y68";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y69";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y68";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y69";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y68";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y69";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y68";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y69";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y72";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y73";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y72";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y73";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y74";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y75";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y74";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y75";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y72";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;
  signal clk_delay_6 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_B_IOB_A
--  Description : DDR Port B Data, Byte 0

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_B_IOB_A is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_B_IOB_A;


architecture RTL of HE_DDR_B_IOB_A is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y7";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y6";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y7";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y6";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y6";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y4";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y5";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y4";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y5";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y4";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y5";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y4";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y5";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y8";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y9";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y8";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y9";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y10";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y11";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y10";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y11";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y8";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_B_IOB_B
--  Description : DDR Port B Data, Byte 1

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_B_IOB_B is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_B_IOB_B;


architecture RTL of HE_DDR_B_IOB_B is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y15";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y14";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y15";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y14";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y14";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y12";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y13";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y12";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y13";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y12";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y13";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y12";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y13";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y16";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y17";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y16";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y17";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y18";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y19";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y18";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y19";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y16";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_B_IOB_C
--  Description : DDR Port B Data, Byte 2

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_B_IOB_C is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_B_IOB_C;


architecture RTL of HE_DDR_B_IOB_C is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y23";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y22";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y23";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y22";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y22";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y20";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y21";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y20";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y21";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y20";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y21";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y20";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y21";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y24";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y25";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y24";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y25";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y26";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y27";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y26";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y27";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y24";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_B_IOB_D
--  Description : DDR Port B Data, Byte 3

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_unsigned.all;


-- pragma translate_off
library UNISIM;
use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on


entity HE_DDR_B_IOB_D is
  port (
    reset              : in  std_logic;
    wclk               : in  std_logic;
    dqs                : in  std_logic;
    write_en_val       : in  std_logic;
    read_en_val        : in  std_logic;
    write_data_rising  : in  std_logic_vector(7 downto 0);
    write_data_falling : in  std_logic_vector(7 downto 0);
    read_data_rising   : out std_logic_vector(7 downto 0);
    read_data_falling  : out std_logic_vector(7 downto 0);
    DELAY_SELECT       : in  std_logic_vector(7 downto 0);
    ddr_data_in        : in  std_logic_vector(7 downto 0);
    ddr_data_out       : out std_logic_vector(7 downto 0);
    ddr_data_en        : out std_logic_vector(7 downto 0)
  );
end HE_DDR_B_IOB_D;


architecture RTL of HE_DDR_B_IOB_D is

  component FD
    port(
      Q   : out std_logic;
      D   : in  std_logic;
      C   : in  std_logic );
  end component;

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDRE
    port (
      D  : in  std_logic;
      R  : in  std_logic;
      C  : in  std_logic;
      Q  : out std_logic;
      CE : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  component MUXF5
    port (
      O  : out std_logic;
      I0 : in  std_logic;
      I1 : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute LOC : string;
  attribute IOB : string;

  attribute IOB of iOFD_0  : label is "TRUE";
  attribute IOB of iOFD_1  : label is "TRUE";
  attribute IOB of iOFD_2  : label is "TRUE";
  attribute IOB of iOFD_3  : label is "TRUE";
  attribute IOB of iOFD_4  : label is "TRUE";
  attribute IOB of iOFD_5  : label is "TRUE";
  attribute IOB of iOFD_6  : label is "TRUE";
  attribute IOB of iOFD_7  : label is "TRUE";

  attribute IOB of iOEN_0  : label is "TRUE";
  attribute IOB of iOEN_1  : label is "TRUE";
  attribute IOB of iOEN_2  : label is "TRUE";
  attribute IOB of iOEN_3  : label is "TRUE";
  attribute IOB of iOEN_4  : label is "TRUE";
  attribute IOB of iOEN_5  : label is "TRUE";
  attribute IOB of iOEN_6  : label is "TRUE";
  attribute IOB of iOEN_7  : label is "TRUE";

  attribute IOB of iIFDR_0 : label is "TRUE";
  attribute IOB of iIFDR_1 : label is "TRUE";
  attribute IOB of iIFDR_2 : label is "TRUE";
  attribute IOB of iIFDR_3 : label is "TRUE";
  attribute IOB of iIFDR_4 : label is "TRUE";
  attribute IOB of iIFDR_5 : label is "TRUE";
  attribute IOB of iIFDR_6 : label is "TRUE";
  attribute IOB of iIFDR_7 : label is "TRUE";

  attribute IOB of iIFDF_0 : label is "TRUE";
  attribute IOB of iIFDF_1 : label is "TRUE";
  attribute IOB of iIFDF_2 : label is "TRUE";
  attribute IOB of iIFDF_3 : label is "TRUE";
  attribute IOB of iIFDF_4 : label is "TRUE";
  attribute IOB of iIFDF_5 : label is "TRUE";
  attribute IOB of iIFDF_6 : label is "TRUE";
  attribute IOB of iIFDF_7 : label is "TRUE";

  attribute LOC of iDELAY_MUX0 : label is "SLICE_X67Y31";
  attribute LOC of iDELAY_MUX1 : label is "SLICE_X67Y30";
  attribute LOC of iDELAY_MUX2 : label is "SLICE_X66Y31";
  attribute LOC of iDELAY_MUX3 : label is "SLICE_X66Y30";
--attribute LOC of iDELAY_MUX4 : label is "SLICE_X64Y30";

  attribute LOC of iDQR_0 : label is "SLICE_X66Y28";
  attribute LOC of iDQF_0 : label is "SLICE_X66Y29";

  attribute LOC of iDQR_1 : label is "SLICE_X66Y28";
  attribute LOC of iDQF_1 : label is "SLICE_X66Y29";

  attribute LOC of iDQR_2 : label is "SLICE_X67Y28";
  attribute LOC of iDQF_2 : label is "SLICE_X67Y29";

  attribute LOC of iDQR_3 : label is "SLICE_X67Y28";
  attribute LOC of iDQF_3 : label is "SLICE_X67Y29";

  attribute LOC of iDQR_4 : label is "SLICE_X67Y32";
  attribute LOC of iDQF_4 : label is "SLICE_X67Y33";

  attribute LOC of iDQR_5 : label is "SLICE_X67Y32";
  attribute LOC of iDQF_5 : label is "SLICE_X67Y33";

  attribute LOC of iDQR_6 : label is "SLICE_X66Y34";
  attribute LOC of iDQF_6 : label is "SLICE_X66Y35";

  attribute LOC of iDQR_7 : label is "SLICE_X66Y34";
  attribute LOC of iDQF_7 : label is "SLICE_X66Y35";

  attribute LOC of iRD_EN_DL : label is "SLICE_X66Y32";

  signal delay_reg   : std_logic_vector(7 downto 0);

  signal GND         : std_logic;
  signal VCC         : std_logic;

  signal clk_delay_1 : std_logic;
  signal clk_delay_2 : std_logic;
  signal clk_delay_3 : std_logic;
  signal clk_delay_4 : std_logic;
  signal clk_delay_5 : std_logic;

  signal wclkb           : std_logic;
  signal clk_rx          : std_logic;
  signal clk_rx_b        : std_logic;
  signal read_en_val_dl1 : std_logic;
  signal read_en_ifd_ce  : std_logic;

  signal rising_data  : std_logic_vector(7 downto 0);
  signal falling_data : std_logic_vector(7 downto 0);

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  clk_rx <= clk_delay_1;

  clk_rx_b <= not clk_rx;

  iRD_EN_DL : FDCE
    port map (
      D   => read_en_val,
      CLR => reset,
      C   => wclkb,
      Q   => read_en_val_dl1,
      CE  => VCC );

  read_en_ifd_ce <= read_en_val OR read_en_val_dl1;

  iDEL0  : FD port map (Q=>delay_reg(0), D=>DELAY_SELECT(0), C=>wclk);
  iDEL1  : FD port map (Q=>delay_reg(1), D=>DELAY_SELECT(1), C=>wclk);
  iDEL2  : FD port map (Q=>delay_reg(2), D=>DELAY_SELECT(2), C=>wclk);
  iDEL3  : FD port map (Q=>delay_reg(3), D=>DELAY_SELECT(3), C=>wclk);
  iDEL4  : FD port map (Q=>delay_reg(4), D=>DELAY_SELECT(4), C=>wclk);
  iDEL5  : FD port map (Q=>delay_reg(5), D=>DELAY_SELECT(5), C=>wclk);
  iDEL6  : FD port map (Q=>delay_reg(6), D=>DELAY_SELECT(6), C=>wclk);
  iDEL7  : FD port map (Q=>delay_reg(7), D=>DELAY_SELECT(7), C=>wclk);

  iDELAY_MUX0 : MUXF5
    port map (
      O  => clk_delay_1,
      I0 => GND,
      I1 => clk_delay_2,
      S  => delay_reg(0) );

  iDELAY_MUX1 : MUXF5
    port map (
      O  => clk_delay_2,
      I0 => GND,
      I1 => clk_delay_3,
      S  => delay_reg(1) );

  iDELAY_MUX2 : MUXF5
    port map (
      O  => clk_delay_3,
      I0 => GND,
      I1 => clk_delay_4,
      S  => delay_reg(2) );

  iDELAY_MUX3 : MUXF5
    port map (
      O  => clk_delay_4,
      I0 => dqs,
      I1 => GND,
      S  => delay_reg(3) );

--iDELAY_MUX4 : MUXF5
--  port map (
--    O  => clk_delay_5,
--    I0 => dqs,
--    I1 => GND,
--    S  => delay_reg(4) );

  iOFD_0 : FDDRRSE port map (Q=>ddr_data_out(0), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(0), D1=>write_data_falling(0), R=>GND, S=>GND);
  iOFD_1 : FDDRRSE port map (Q=>ddr_data_out(1), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(1), D1=>write_data_falling(1), R=>GND, S=>GND);
  iOFD_2 : FDDRRSE port map (Q=>ddr_data_out(2), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(2), D1=>write_data_falling(2), R=>GND, S=>GND);
  iOFD_3 : FDDRRSE port map (Q=>ddr_data_out(3), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(3), D1=>write_data_falling(3), R=>GND, S=>GND);
  iOFD_4 : FDDRRSE port map (Q=>ddr_data_out(4), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(4), D1=>write_data_falling(4), R=>GND, S=>GND);
  iOFD_5 : FDDRRSE port map (Q=>ddr_data_out(5), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(5), D1=>write_data_falling(5), R=>GND, S=>GND);
  iOFD_6 : FDDRRSE port map (Q=>ddr_data_out(6), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(6), D1=>write_data_falling(6), R=>GND, S=>GND);
  iOFD_7 : FDDRRSE port map (Q=>ddr_data_out(7), C0=>wclk, C1=>wclkb, CE=>VCC, D0=>write_data_rising(7), D1=>write_data_falling(7), R=>GND, S=>GND);

  iOEN_0 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(0), CE=>VCC);
  iOEN_1 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(1), CE=>VCC);
  iOEN_2 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(2), CE=>VCC);
  iOEN_3 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(3), CE=>VCC);
  iOEN_4 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(4), CE=>VCC);
  iOEN_5 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(5), CE=>VCC);
  iOEN_6 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(6), CE=>VCC);
  iOEN_7 : FDCE port map (D=>write_en_val, CLR=>reset, C=>wclk, Q=>ddr_data_en(7), CE=>VCC);

  iIFDR_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx, Q=>rising_data(0), CE=>read_en_ifd_ce);
  iIFDR_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx, Q=>rising_data(1), CE=>read_en_ifd_ce);
  iIFDR_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx, Q=>rising_data(2), CE=>read_en_ifd_ce);
  iIFDR_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx, Q=>rising_data(3), CE=>read_en_ifd_ce);
  iIFDR_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx, Q=>rising_data(4), CE=>read_en_ifd_ce);
  iIFDR_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx, Q=>rising_data(5), CE=>read_en_ifd_ce);
  iIFDR_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx, Q=>rising_data(6), CE=>read_en_ifd_ce);
  iIFDR_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx, Q=>rising_data(7), CE=>read_en_ifd_ce);

  iIFDF_0 : FDRE port map (D=>ddr_data_in(0), R=>GND, C=>clk_rx_b, Q=>falling_data(0), CE=>read_en_ifd_ce);
  iIFDF_1 : FDRE port map (D=>ddr_data_in(1), R=>GND, C=>clk_rx_b, Q=>falling_data(1), CE=>read_en_ifd_ce);
  iIFDF_2 : FDRE port map (D=>ddr_data_in(2), R=>GND, C=>clk_rx_b, Q=>falling_data(2), CE=>read_en_ifd_ce);
  iIFDF_3 : FDRE port map (D=>ddr_data_in(3), R=>GND, C=>clk_rx_b, Q=>falling_data(3), CE=>read_en_ifd_ce);
  iIFDF_4 : FDRE port map (D=>ddr_data_in(4), R=>GND, C=>clk_rx_b, Q=>falling_data(4), CE=>read_en_ifd_ce);
  iIFDF_5 : FDRE port map (D=>ddr_data_in(5), R=>GND, C=>clk_rx_b, Q=>falling_data(5), CE=>read_en_ifd_ce);
  iIFDF_6 : FDRE port map (D=>ddr_data_in(6), R=>GND, C=>clk_rx_b, Q=>falling_data(6), CE=>read_en_ifd_ce);
  iIFDF_7 : FDRE port map (D=>ddr_data_in(7), R=>GND, C=>clk_rx_b, Q=>falling_data(7), CE=>read_en_ifd_ce);

  iDQR_0 : FDRE port map (D=>rising_data(0), R=>GND, C=>wclk, Q=>read_data_rising(0), CE=>read_en_val);
  iDQR_1 : FDRE port map (D=>rising_data(1), R=>GND, C=>wclk, Q=>read_data_rising(1), CE=>read_en_val);
  iDQR_2 : FDRE port map (D=>rising_data(2), R=>GND, C=>wclk, Q=>read_data_rising(2), CE=>read_en_val);
  iDQR_3 : FDRE port map (D=>rising_data(3), R=>GND, C=>wclk, Q=>read_data_rising(3), CE=>read_en_val);
  iDQR_4 : FDRE port map (D=>rising_data(4), R=>GND, C=>wclk, Q=>read_data_rising(4), CE=>read_en_val);
  iDQR_5 : FDRE port map (D=>rising_data(5), R=>GND, C=>wclk, Q=>read_data_rising(5), CE=>read_en_val);
  iDQR_6 : FDRE port map (D=>rising_data(6), R=>GND, C=>wclk, Q=>read_data_rising(6), CE=>read_en_val);
  iDQR_7 : FDRE port map (D=>rising_data(7), R=>GND, C=>wclk, Q=>read_data_rising(7), CE=>read_en_val);

  iDQF_0 : FDRE port map (D=>falling_data(0), R=>GND, C=>wclkb, Q=>read_data_falling(0), CE=>read_en_val_dl1);
  iDQF_1 : FDRE port map (D=>falling_data(1), R=>GND, C=>wclkb, Q=>read_data_falling(1), CE=>read_en_val_dl1);
  iDQF_2 : FDRE port map (D=>falling_data(2), R=>GND, C=>wclkb, Q=>read_data_falling(2), CE=>read_en_val_dl1);
  iDQF_3 : FDRE port map (D=>falling_data(3), R=>GND, C=>wclkb, Q=>read_data_falling(3), CE=>read_en_val_dl1);
  iDQF_4 : FDRE port map (D=>falling_data(4), R=>GND, C=>wclkb, Q=>read_data_falling(4), CE=>read_en_val_dl1);
  iDQF_5 : FDRE port map (D=>falling_data(5), R=>GND, C=>wclkb, Q=>read_data_falling(5), CE=>read_en_val_dl1);
  iDQF_6 : FDRE port map (D=>falling_data(6), R=>GND, C=>wclkb, Q=>read_data_falling(6), CE=>read_en_val_dl1);
  iDQF_7 : FDRE port map (D=>falling_data(7), R=>GND, C=>wclkb, Q=>read_data_falling(7), CE=>read_en_val_dl1);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_DQS
--  Description : DDR Data Strobe

library IEEE;
  use IEEE.std_logic_1164.all;

-- pragma translate_off
library UNISIM;
  use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity HE_DDR_DQS is
  port (
    DQS_enable  : in  std_logic;
    DQS_reset   : in  std_logic;
    clk         : in  std_logic;
    clk180      : in  std_logic;
    DDR_DQS_IN  : in  std_logic_vector(3 downto 0);
    DDR_DQS_OUT : out std_logic_vector(3 downto 0);
    DDR_DQS_EN  : out std_logic_vector(3 downto 0);
    DQS         : out std_logic_vector(3 downto 0)
  );
end HE_DDR_DQS;


architecture RTL of HE_DDR_DQS is

  component FD
    port(
      Q : out std_logic;
      D : in  std_logic;
      C : in  std_logic
    );
  end component;

  component FDDRRSE
    port(
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  attribute IOB : string;

  attribute IOB of iOEN_0 : label is "TRUE";
  attribute IOB of iOEN_1 : label is "TRUE";
  attribute IOB of iOEN_2 : label is "TRUE";
  attribute IOB of iOEN_3 : label is "TRUE";

  attribute IOB of iOFD_0 : label is "TRUE";
  attribute IOB of iOFD_1 : label is "TRUE";
  attribute IOB of iOFD_2 : label is "TRUE";
  attribute IOB of iOFD_3 : label is "TRUE";

  signal GND           : std_logic;
  signal VCC           : std_logic;

  signal DQS_reset_rg  : std_logic := '0';
  signal DQS_enable_rg : std_logic := '0';

begin

  VCC <= '1';
  GND <= '0';

  DQS(0) <= DDR_DQS_IN(0);
  DQS(1) <= DDR_DQS_IN(1);
  DQS(2) <= DDR_DQS_IN(2);
  DQS(3) <= DDR_DQS_IN(3);

  process(clk)
  begin
    if clk'event and clk='1' then
      DQS_reset_rg  <= DQS_reset;
      DQS_enable_rg <= DQS_enable;
    end if;
  end process;

  iOFD_0 : FDDRRSE
    port map (
      Q  => DDR_DQS_OUT(0),
      C0 => clk,
      C1 => clk180,
      CE => VCC,
      D0 => VCC,
      D1 => GND,
      R  => DQS_reset_rg,
      S  => GND );

  iOFD_1 : FDDRRSE
    port map (
      Q  => DDR_DQS_OUT(1),
      C0 => clk,
      C1 => clk180,
      CE => VCC,
      D0 => VCC,
      D1 => GND,
      R  => DQS_reset_rg,
      S  => GND );

  iOFD_2 : FDDRRSE
    port map (
      Q  => DDR_DQS_OUT(2),
      C0 => clk,
      C1 => clk180,
      CE => VCC,
      D0 => VCC,
      D1 => GND,
      R  => DQS_reset_rg,
      S  => GND );

  iOFD_3 : FDDRRSE
    port map (
      Q  => DDR_DQS_OUT(3),
      C0 => clk,
      C1 => clk180,
      CE => VCC,
      D0 => VCC,
      D1 => GND,
      R  => DQS_reset_rg,
      S  => GND );

  iOEN_0 : FD port map (D=>DQS_enable_rg, Q=>DDR_DQS_EN(0), C=>clk);
  iOEN_1 : FD port map (D=>DQS_enable_rg, Q=>DDR_DQS_EN(1), C=>clk);
  iOEN_2 : FD port map (D=>DQS_enable_rg, Q=>DDR_DQS_EN(2), C=>clk);
  iOEN_3 : FD port map (D=>DQS_enable_rg, Q=>DDR_DQS_EN(3), C=>clk);

end RTL;


-- ==============================================================
--  Module      : HE_DDR_DM_IOB
--  Description : DDR Data Mask

library IEEE;
  use IEEE.std_logic_1164.all;

-- pragma translate_off
library UNISIM;
  use UNISIM.VCOMPONENTS.ALL;
-- pragma translate_on

entity HE_DDR_DM_IOB is
  port (
    write_data_mask_rising  : in  std_logic;
    write_data_mask_falling : in  std_logic;
    wclk                    : in  std_logic;
    reset                   : in  std_logic;
    write_en_val            : in  std_logic;
    ddr_dm_out              : out std_logic;
    ddr_dm_en               : out std_logic
  );
end HE_DDR_DM_IOB;


architecture RTL of HE_DDR_DM_IOB is

  component FDDRRSE
    port (
      Q  : out std_logic;
      C0 : in  std_logic;
      C1 : in  std_logic;
      CE : in  std_logic;
      D0 : in  std_logic;
      D1 : in  std_logic;
      R  : in  std_logic;
      S  : in  std_logic
    );
  end component;

  component FDCE
    port (
      D   : in  std_logic;
      CLR : in  std_logic;
      C   : in  std_logic;
      Q   : out std_logic;
      CE  : in  std_logic
    );
  end component;

  attribute IOB : string;

  attribute IOB of DQ_T    : label is "TRUE";
  attribute IOB of DDR_OUT : label is "TRUE";

  signal GND   : std_logic;
  signal VCC   : std_logic;

  signal wclkb : std_logic;

begin

  GND <= '0';
  VCC <= '1';

  wclkb <= not wclk;

  DDR_OUT : FDDRRSE
    port map (
      Q  => ddr_dm_out,
      C0 => wclk,
      C1 => wclkb,
      CE => VCC,
      D0 => write_data_mask_rising,
      D1 => write_data_mask_falling,
      R  => GND,
      S  => GND );

  DQ_T : FDCE
    port map (
      D   => write_en_val,
      CLR => reset,
      C   => wclk,
      Q   => ddr_dm_en,
      CE  => VCC );

end RTL;
