

/* Uncomment the following line to use instruction cache */
#define USE_CACHE


/* Xilinx provided header file for cache functions */
#ifdef USE_CACHE
#include "xcache_l.h"
#endif


/* Xilinx provided header file for timer functions */
#include "xtime_l.h"


#define BANK_SIZE 33554432
#define HALF_SIZE 16777216


void Send_Word_to_Host(unsigned long data);


/* DDR Bank A Addresses */
volatile unsigned char  *ddr_a_bytes     = (unsigned char  *)0x80000000;
volatile unsigned short *ddr_a_halfwords = (unsigned short *)0x80000000;
volatile unsigned long  *ddr_a_words     = (unsigned long  *)0x80000000;

/* DDR Bank B Addresses*/
volatile unsigned char  *ddr_b_bytes     = (unsigned char  *)0x88000000;
volatile unsigned short *ddr_b_halfwords = (unsigned short *)0x88000000;
volatile unsigned long  *ddr_b_words     = (unsigned long  *)0x88000000;

/* OPB2HSB Addresses */
volatile unsigned long  *hsb_data_rg     = (unsigned long  *)0x40000000;
volatile unsigned long  *hsb_count_rg    = (unsigned long  *)0x40000004;
volatile unsigned long  *hsb_flags_rg    = (unsigned long  *)0x40000008;



int main()
{
int i;
int loops;
int errors;
XTime Write_Start;
XTime Write_Stop;
XTime Write_Total;
XTime Read_Start;
XTime Read_Stop;
XTime Read_Total;


#ifdef USE_CACHE
XCache_EnableICache((unsigned int)0x00000001);
XCache_WriteCCR0((unsigned int)0x00700a00);
#endif


loops = 0;
errors = 0;
while (1) {
	Write_Total = 0;
	Read_Total  = 0;

	// Test with 0
	XTime_GetTime(&Write_Start);
	for (i=0;i<HALF_SIZE;i++) ddr_a_words[HALF_SIZE + i] = 0x00000000;
	XTime_GetTime(&Write_Stop);
	Write_Total += (Write_Stop - Write_Start);

	XTime_GetTime(&Read_Start);
	for (i=0;i<HALF_SIZE;i++) {
		if (ddr_a_words[HALF_SIZE + i] != 0x00000000) errors++;
	}
	XTime_GetTime(&Read_Stop);
	Read_Total += (Read_Stop - Read_Start);

	// Test with 1
	XTime_GetTime(&Write_Start);
	for (i=0;i<HALF_SIZE;i++) ddr_a_words[HALF_SIZE + i] = 0xffffffff;
	XTime_GetTime(&Write_Stop);
	Write_Total += (Write_Stop - Write_Start);

	XTime_GetTime(&Read_Start);
	for (i=0;i<HALF_SIZE;i++) {
		if (ddr_a_words[HALF_SIZE + i] != 0xffffffff) errors++;
	}
	XTime_GetTime(&Read_Stop);
	Read_Total += (Read_Stop - Read_Start);

	// Test with 5
	XTime_GetTime(&Write_Start);
	for (i=0;i<HALF_SIZE;i++) ddr_a_words[HALF_SIZE + i] = 0x55555555;
	XTime_GetTime(&Write_Stop);
	Write_Total += (Write_Stop - Write_Start);

	XTime_GetTime(&Read_Start);
	for (i=0;i<HALF_SIZE;i++) {
		if (ddr_a_words[HALF_SIZE + i] != 0x55555555) errors++;
	}
	XTime_GetTime(&Read_Stop);
	Read_Total += (Read_Stop - Read_Start);

	// Test with A
	XTime_GetTime(&Write_Start);
	for (i=0;i<HALF_SIZE;i++) ddr_a_words[HALF_SIZE + i] = 0xaaaaaaaa;
	XTime_GetTime(&Write_Stop);
	Write_Total += (Write_Stop - Write_Start);

	XTime_GetTime(&Read_Start);
	for (i=0;i<HALF_SIZE;i++) {
		if (ddr_a_words[HALF_SIZE + i] != 0xaaaaaaaa) errors++;
	}
	XTime_GetTime(&Read_Stop);
	Read_Total += (Read_Stop - Read_Start);

	// Test with Address
	XTime_GetTime(&Write_Start);
	for (i=0;i<HALF_SIZE;i++) ddr_a_words[HALF_SIZE + i] = i;
	XTime_GetTime(&Write_Stop);
	Write_Total += (Write_Stop - Write_Start);

	XTime_GetTime(&Read_Start);
	for (i=0;i<HALF_SIZE;i++) {
		if (ddr_a_words[HALF_SIZE + i] != i) errors++;
	}
	XTime_GetTime(&Read_Stop);
	Read_Total += (Read_Stop - Read_Start);

	loops++;

	Send_Word_to_Host(loops);
	Send_Word_to_Host(errors);

	Send_Word_to_Host(Write_Total);
	Send_Word_to_Host(Write_Total >> 32);

	Send_Word_to_Host(Read_Total);
	Send_Word_to_Host(Read_Total >> 32);
}

return(0);
}


void Send_Word_to_Host(unsigned long data)
{
	int hsb_count;

	do hsb_count = *hsb_count_rg;
	while ((hsb_count & 0x0f) > 11);

	*hsb_data_rg = (data & 0x000000ff);
	*hsb_data_rg = (data & 0x0000ff00) >> 8;
	*hsb_data_rg = (data & 0x00ff0000) >> 16;
	*hsb_data_rg = (data & 0xff000000) >> 24;

	return;
}
