--------------------------------------------------------------------------------

-- Copyright (c) 1996-2004 Xilinx, Inc.
-- All Rights Reserved
--------------------------------------------------------------------------------

--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.00.a
--  \   \        Filename: opb_ipif_ssp1.vhd
--  /   /        Date Last Modified:  05/23/2003
-- /___/   /\    Date Created: 06/18/2003
-- \   \  /  \
--  \___\/\___\
-- 
-- Device: Xilinx
-- Library: opb_gpio_v3_01_a
-- Purpose: ipif interface
-- Reference:
--    Synthesis and Verification Design Guide, Chapter 6
-- Revision History:
-- Description:     OPB IPIF Slave Services Package 1
--                  This packages configures OPB IPIF to include the following
--                  services:
--                      -- OPB slave attachment
--                      -- Address decode
--                      -- Module Identification Register (MIR)/Reset register
--                      -- IP Interrupt Controller                   
--
-------------------------------------------------------------------------------
-- Structure:       opb_ipif_ssp1
--                      --opb_ipif
--                          -- opb_bam
--                          -- reset_mir
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ipif_common_v1_00_c;
use ipif_common_v1_00_c.ipif_pkg.all;

library opb_ipif_v3_00_a;
use opb_ipif_v3_00_a.all;
-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Definition of Generics:
--      C_BASEADDR                 -- User logic base address                                                                                 
--      C_HIGHADDR                 -- User logic high address
--      C_MIR_BASEADDR             -- Base address of MIR/Reset register
--      C_MIR_HIGHADDR             -- High address of MIR/Reset register
--      C_INTR_BASEADDR            -- Base address of ISC
--      C_INTR_HIGHADDR            -- High address of ISC
--      C_USER_ID_CODE             -- User ID to place in MIR/Reset register
--      C_PIPELINE_MODEL           -- Bus interface pipeline model
--      C_NUM_USER_INTR            -- number of user interrupts
--      C_USER_INTR_TYPE           -- user interrupt type (pass thru, reg_event, edge_detect)
--      C_OPB_DWIDTH               -- OPB data bus width
--      C_OPB_AWIDTH               -- OPB address bus width
--      C_FAMILY                   -- Target FPGA architecture
--
-- Definition of Ports:
--  OPB Bus
--      OPB_ABus                   -- OPB address bus                                             
--      OPB_DBus                   -- OPB  data bus                                                   
--      Sln_DBus                   -- Slave read bus                                         
--      OPB_select                 -- OPB Select                                    
--      OPB_RNW                    -- OPB read not write                                           
--      OPB_seqAddr                -- OPB sequential address                              
--      OPB_BE                     -- OPB byte enables                                              
--      Sln_xferAck                -- Slave transfer acknowledge                            
--      Sln_errAck                 -- Slave Error acknowledge
--      Sln_toutSup                -- Slave Timeout Suppress
--      Sln_retry                  -- Slave retry
-- IPIC
--      Bus2IP_Addr                 -- Bus to IP address        
--      Bus2IP_BE                   -- Bus to IP byte enables
--      Bus2IP_Burst                -- Bus to IP burst
--      Bus2IP_Clk                  -- Bus to IP clock
--      Bus2IP_CS                   -- Bus to IP chip select
--      Bus2IP_Data                 -- Bus to IP data bus
--      Bus2IP_RdCE                 -- Bus to IP read chip enable
--      Bus2IP_Reset                -- Bus to IP reset
--      Bus2IP_RNW                  -- Bus to IP read not write
--      Bus2IP_WrCE                 -- Bus to IP write chip enable
--      IP2Bus_Ack                  -- IP to Bus transfer acknowledge
--      IP2Bus_Data                 -- IP to Bus data bus
--      IP2Bus_Error                -- IP to Bus error acknowledge
--      IP2Bus_PostedWrInh          -- IP to Bus posted write inhibit
--      IP2Bus_Retry                -- IP to Bus retry
--      IP2Bus_ToutSup              -- IP to Bus timeout suppress
-- Interrupts
--      IP2Bus_Intr                -- IP generated interrupts
--      IP2INTC_Irpt                -- Interrupt output to processor or ISC
--
-----------------------------------------------------------------------------
-- Entity section
-----------------------------------------------------------------------------
entity opb_ipif_ssp1 is
  generic
  (
    C_BASEADDR         : std_logic_vector(0 to 31) := X"FFFE_0400";
    C_HIGHADDR         : std_logic_vector(0 to 31) := X"FFFE_04FF";
  
    C_INTR_BASEADDR    : std_logic_vector(0 to 31) := X"FFFE_0500";
    C_INTR_HIGHADDR    : std_logic_vector(0 to 31) := X"FFFE_05FF";

    C_USER_ID_CODE     : integer                   := 1;
    C_PIPELINE_MODEL   : integer                   := 4;
    
    C_INCLUDE_INTR     : boolean                   := FALSE;
    C_NUM_USER_INTR    : integer                   := 1;
    C_USER_INTR_TYPE   : integer                   := INTR_POS_EDGE_DETECT;

    C_OPB_AWIDTH       : integer                   := 32;
    C_OPB_DWIDTH       : integer                   := 32;
    
    C_FAMILY           : string                    := "virtex2"
  );
  port
  (
      OPB_ABus           : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
      OPB_BE             : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      OPB_Clk            : in  std_logic;
      OPB_DBus           : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
      OPB_RNW            : in  std_logic;
      OPB_Rst            : in  std_logic;
      OPB_select         : in  std_logic;
      OPB_seqAddr        : in  std_logic;
      Sln_DBus           : out std_logic_vector(0 to C_OPB_DWIDTH-1);
      Sln_errAck         : out std_logic;
      Sln_retry          : out std_logic;
      Sln_toutSup        : out std_logic;
      Sln_xferAck        : out std_logic;
      Bus2IP_Addr        : out std_logic_vector(0 to C_OPB_AWIDTH-1);
      Bus2IP_BE          : out std_logic_vector(0 to C_OPB_DWIDTH/8-1);
      Bus2IP_Burst       : out std_logic;
      Bus2IP_Clk         : out std_logic;
      Bus2IP_CS          : out std_logic;
      Bus2IP_Data        : out std_logic_vector(0 to C_OPB_DWIDTH-1);
      Bus2IP_RdCE        : out std_logic;
      Bus2IP_Reset       : out std_logic;
      Bus2IP_RNW         : out std_logic;
      Bus2IP_WrCE        : out std_logic;
      IP2Bus_Ack         : in  std_logic;
      IP2Bus_Data        : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
      IP2Bus_Error       : in  std_logic;
      IP2Bus_PostedWrInh : in  std_logic;
      IP2Bus_Retry       : in  std_logic;
      IP2Bus_ToutSup     : in  std_logic;
      -- Interrupts
      IP2Bus_Intr        : in  std_logic_vector(0 to C_NUM_USER_INTR-1);
      IP2INTC_Irpt        : out std_logic
  );
      --fan-out attributes for XST
      attribute MAX_FANOUT                  : string;
      attribute MAX_FANOUT   of OPB_Clk     : signal is "10000";
      attribute MAX_FANOUT   of OPB_Rst     : signal is "10000";


end entity opb_ipif_ssp1;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture imp of opb_ipif_ssp1 is

constant ZERO_ADDR_PAD       :  std_logic_vector(0 to 31) := (others => '0');
-------------------------------------------------------------------------------
-- Function definitions
-------------------------------------------------------------------------------
function populate_intr_mode_array (num_user_intr        : integer;
                                   intr_capture_mode    : integer)
                                   return INTEGER_ARRAY_TYPE is
    variable intr_mode_array    : INTEGER_ARRAY_TYPE(0 to num_user_intr-1);
begin
    for i in 0 to num_user_intr-1 loop
        intr_mode_array(i) := intr_capture_mode;
    end loop;
    
    return intr_mode_array;
end function populate_intr_mode_array;


function add_intr_ard_id_array(include_intr    : boolean;
                              ard_id_array     : INTEGER_ARRAY_TYPE)
                              return INTEGER_ARRAY_TYPE is
    variable intr_ard_id_array   : INTEGER_ARRAY_TYPE(0 to ard_id_array'length);
begin
    intr_ard_id_array(0 to ard_id_array'length-1) := ard_id_array;
    if include_intr then
       intr_ard_id_array(ard_id_array'length) := IPIF_INTR;
       return intr_ard_id_array;
    else
        return ard_id_array;
    end if;
end function add_intr_ard_id_array;


function add_intr_ard_addr_range_array(include_intr    : boolean;
                              intr_baseaddr            : std_logic_vector;
                              intr_highaddr            : std_logic_vector;
                              ard_id_array             : INTEGER_ARRAY_TYPE;
                              ard_addr_range_array     : SLV64_ARRAY_TYPE)
                              return SLV64_ARRAY_TYPE is
    variable intr_ard_addr_range_array   : SLV64_ARRAY_TYPE(0 to ard_addr_range_array'length+1);
begin
    intr_ard_addr_range_array(0 to ard_addr_range_array'length-1) := ard_addr_range_array;
    if include_intr  then
       intr_ard_addr_range_array(2*get_id_index(ard_id_array,IPIF_INTR)) 
                        := ZERO_ADDR_PAD & intr_baseaddr;
       intr_ard_addr_range_array(2*get_id_index(ard_id_array,IPIF_INTR)+1) 
                        := ZERO_ADDR_PAD & intr_highaddr;
       return intr_ard_addr_range_array;
    else
        return ard_addr_range_array;
    end if;
end function add_intr_ard_addr_range_array;

function add_intr_ard_dwidth_array(include_intr    : boolean;
                              intr_dwidth           : integer;
                              ard_id_array          : INTEGER_ARRAY_TYPE;
                              ard_dwidth_array      : INTEGER_ARRAY_TYPE)
                              return INTEGER_ARRAY_TYPE is
    variable intr_ard_dwidth_array   : INTEGER_ARRAY_TYPE(0 to ard_dwidth_array'length);
begin
    intr_ard_dwidth_array(0 to ard_dwidth_array'length-1) := ard_dwidth_array;
    if include_intr  then
       intr_ard_dwidth_array(get_id_index(ard_id_array, IPIF_INTR)) := intr_dwidth;
       return intr_ard_dwidth_array;
    else
        return ard_dwidth_array;
    end if;
end function add_intr_ard_dwidth_array;


function add_intr_ard_num_ce_array(include_intr     : boolean;
                              ard_id_array          : INTEGER_ARRAY_TYPE;
                              ard_num_ce_array      : INTEGER_ARRAY_TYPE)
                              return INTEGER_ARRAY_TYPE is
    variable intr_ard_num_ce_array   : INTEGER_ARRAY_TYPE(0 to ard_num_ce_array'length);
begin
    intr_ard_num_ce_array(0 to ard_num_ce_array'length-1) := ard_num_ce_array;
    if include_intr  then
       intr_ard_num_ce_array(get_id_index(ard_id_array, IPIF_INTR)) := 16;
       return intr_ard_num_ce_array;
    else
        return ard_num_ce_array;
    end if;
end function add_intr_ard_num_ce_array;


-------------------------------------------------------------------------------
-- Constant Declarations
-------------------------------------------------------------------------------

constant USER_ARD_ID_ARRAY : INTEGER_ARRAY_TYPE :=
        (
           0 => USER_00     
         );

constant ARD_ID_ARRAY           : INTEGER_ARRAY_TYPE := 
         add_intr_ard_id_array(C_INCLUDE_INTR,USER_ARD_ID_ARRAY);

constant USER_ADDR_RANGE_ARRAY  : SLV64_ARRAY_TYPE :=
        ( 0 => ZERO_ADDR_PAD & C_BASEADDR,
          1 => ZERO_ADDR_PAD & C_HIGHADDR
         );
constant ARD_ADDR_RANGE_ARRAY   : SLV64_ARRAY_TYPE :=
                     add_intr_ard_addr_range_array
                        (C_INCLUDE_INTR,
                         C_INTR_BASEADDR,
                         C_INTR_HIGHADDR,
                         ARD_ID_ARRAY,
                             USER_ADDR_RANGE_ARRAY);
                                                                    
constant USER_DWIDTH_ARRAY      : INTEGER_ARRAY_TYPE :=
         ( 0 => 32
         );
constant ARD_DWIDTH_ARRAY       : INTEGER_ARRAY_TYPE :=
                     add_intr_ard_dwidth_array
                        (C_INCLUDE_INTR,
                         C_OPB_DWIDTH,
                         ARD_ID_ARRAY,
                             USER_DWIDTH_ARRAY);
                             
constant USER_NUM_CE_ARRAY    : INTEGER_ARRAY_TYPE :=
        ( 0 => 1
        );
constant ARD_NUM_CE_ARRAY     : INTEGER_ARRAY_TYPE :=
                 add_intr_ard_num_ce_array
                    (C_INCLUDE_INTR,
                     ARD_ID_ARRAY,
                         USER_NUM_CE_ARRAY);

constant ARD_DEPENDENT_PROPS_ARRAY : DEPENDENT_PROPS_ARRAY_TYPE := 
        (0 => (others => 0),
         1 => (others => 0)
        );

-- Set the pipeline model
-- 1 = include OPB-In pipeline registers
-- 2 = include IP pipeline registers
-- 3 = include OPB-In and IP pipeline registers
-- 4 = include OPB-Out pipeline registers
-- 5 = include OPB-In and OPB-Out pipeline registers
-- 6 = include IP and OPB-Out pipeline registers
-- 7 = include OPB-In, IP, and OPB-Out pipeline registers
constant PIPELINE_MODEL         : integer := C_PIPELINE_MODEL; 

-- Include IP interrupts, don't need Device ISC or IID
constant INCLUDE_DEV_ISC        : integer := 0;
constant INCLUDE_DEV_IID        : integer := 0;
constant IP_INTR_MODE_ARRAY     : integer_array_type := 
         populate_intr_mode_array(C_NUM_USER_INTR, C_USER_INTR_TYPE);       

-- No bursting
constant DEV_BURST_ENABLE       : integer := 0;

-- Index for user logic
constant USER00_CS_INDEX     : integer := 
                                get_id_index(ARD_ID_ARRAY, USER_00);
constant USER00_CE_INDEX     : integer := 
                        calc_start_ce_index(ARD_NUM_CE_ARRAY,USER00_CS_INDEX);
-------------------------------------------------------------------------------
-- Signal and Type Declarations
-------------------------------------------------------------------------------
-- IPIC Used Signals
signal IP2Bus_Ack_vec         : std_logic_vector(0 to ARD_ID_ARRAY'length-1) 
                                := (others => '0');
signal IP2Bus_Data_vec        : std_logic_vector(0 to C_OPB_DWIDTH*calc_num_ce(ARD_NUM_CE_ARRAY)-1)
                                := (others => '0');
signal IP2Bus_Error_vec       : std_logic_vector(0 to ARD_ID_ARRAY'length-1)
                                := (others => '0');
signal IP2Bus_PostedWrInh_vec : std_logic_vector(0 to ARD_ID_ARRAY'length-1)
                                := (others => '0');
signal IP2Bus_Retry_vec       : std_logic_vector(0 to ARD_ID_ARRAY'length-1)
                                := (others => '0');
signal IP2Bus_ToutSup_vec     : std_logic_vector(0 to ARD_ID_ARRAY'length-1)                             
                                := (others => '0');

signal Bus2IP_CS_vec          : std_logic_vector(0 to ARD_ID_ARRAY'length-1);
signal Bus2IP_RdCE_vec        : std_logic_vector(0 to calc_num_ce(ARD_NUM_CE_ARRAY)-1);
signal Bus2IP_WrCE_vec        : std_logic_vector(0 to calc_num_ce(ARD_NUM_CE_ARRAY)-1);

-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------
component opb_ipif 
  generic
  (
    C_ARD_ID_ARRAY              : INTEGER_ARRAY_TYPE         := ( 0 => IPIF_INTR, 1 => USER_00 );
    C_ARD_ADDR_RANGE_ARRAY      : SLV64_ARRAY_TYPE           := ( x"0000_0000_2000_0000", x"0000_0000_2000_003F", x"0000_0000_2000_0100", x"0000_0000_2000_01FF"  );
    C_ARD_DWIDTH_ARRAY          : INTEGER_ARRAY_TYPE         := ( 32, 32, 32 );
    C_ARD_NUM_CE_ARRAY          : INTEGER_ARRAY_TYPE         := ( 16, 1);
    C_ARD_DEPENDENT_PROPS_ARRAY : DEPENDENT_PROPS_ARRAY_TYPE := ( 0 => (others => 0) ,1 => (others => 0) ,2 => (others => 0) );
    C_PIPELINE_MODEL            : integer                    := 4;
    C_DEV_BLK_ID                : INTEGER                    := 1;
    C_DEV_MIR_ENABLE            : INTEGER                    := 0;
    C_AWIDTH                    : INTEGER                    := 32;
    C_DWIDTH                    : INTEGER                    := 32;
    C_FAMILY                    : string                     := "virtexe";
    C_IP_INTR_MODE_ARRAY        : INTEGER_ARRAY_TYPE         := ( 4, 1 );
    C_INCLUDE_DEV_ISC           : INTEGER                    := 0;
    C_INCLUDE_DEV_IID           : integer                    := 0;
    C_DEV_BURST_ENABLE          : INTEGER                    := 0
  );
  port
  (
    OPB_select         : in  std_logic;
    OPB_DBus           : in  std_logic_vector(0 to C_DWIDTH-1);
    OPB_ABus           : in  std_logic_vector(0 to C_AWIDTH-1);
    OPB_BE             : in  std_logic_vector(0 to C_DWIDTH/8-1);
    OPB_RNW            : in  std_logic;
    OPB_seqAddr        : in  std_logic;
    OPB_xferAck        : in  std_logic;
    Sln_DBus           : out std_logic_vector(0 to C_DWIDTH-1);
    Sln_xferAck        : out std_logic;
    Sln_errAck         : out std_logic;
    Sln_retry          : out std_logic;
    Sln_toutSup        : out std_logic;
    Bus2IP_CS          : out std_logic_vector(0 to C_ARD_ID_ARRAY'length-1);
    Bus2IP_CE          : out std_logic_vector(0 to calc_num_ce(C_ARD_NUM_CE_ARRAY)-1);
    Bus2IP_RdCE        : out std_logic_vector(0 to calc_num_ce(C_ARD_NUM_CE_ARRAY)-1);
    Bus2IP_WrCE        : out std_logic_vector(0 to calc_num_ce(C_ARD_NUM_CE_ARRAY)-1);
    Bus2IP_Data        : out std_logic_vector(0 to C_DWIDTH-1);
    Bus2IP_Addr        : out std_logic_vector(0 to C_AWIDTH-1);
    Bus2IP_BE          : out std_logic_vector(0 to C_DWIDTH/8-1);
    Bus2IP_RNW         : out std_logic;
    Bus2IP_Burst       : out std_logic;
    IP2Bus_Data        : in  std_logic_vector(0 to C_DWIDTH*calc_num_ce(C_ARD_NUM_CE_ARRAY)-1);
    IP2Bus_Ack         : in  std_logic_vector(0 to C_ARD_ID_ARRAY'length-1);
    IP2Bus_Error       : in  std_logic_vector(0 to C_ARD_ID_ARRAY'length-1);
    IP2Bus_Retry       : in  std_logic_vector(0 to C_ARD_ID_ARRAY'length-1);
    IP2Bus_ToutSup     : in  std_logic_vector(0 to C_ARD_ID_ARRAY'length-1);
    IP2Bus_PostedWrInh : in  std_logic_vector(0 to C_ARD_ID_ARRAY'length-1);
    OPB_Clk            : in  std_logic;
    Bus2IP_Clk         : out std_logic;
    IP2Bus_Clk         : in  std_logic;
    Reset              : in  std_logic;
    Bus2IP_Reset       : out std_logic;
    IP2Bus_Intr        : in  std_logic_vector(0 to C_IP_INTR_MODE_ARRAY'length-1);
    Device_Intr        : out std_logic
  );
end component opb_ipif;

-------------------------------------------------------------------------------
begin  
-------------------------------------------------------------------------------
-- assign IP inputs/output to correct vector value
  IP2Bus_Data_vec(USER00_CE_INDEX*C_OPB_DWIDTH to USER00_CE_INDEX*C_OPB_DWIDTH+C_OPB_DWIDTH-1) <= IP2Bus_Data;      
 
  IP2Bus_Ack_vec(USER00_CS_INDEX)         <= IP2Bus_Ack;        
  IP2Bus_Error_vec(USER00_CS_INDEX)       <= IP2Bus_Error;      
  IP2Bus_PostedWrInh_vec(USER00_CS_INDEX) <= IP2Bus_PostedWrInh;
  IP2Bus_Retry_vec(USER00_CS_INDEX)       <= IP2Bus_Retry;   
  IP2Bus_ToutSup_vec(USER00_CS_INDEX)     <= IP2Bus_ToutSup;   

  Bus2IP_CS                 <= Bus2IP_CS_vec(USER00_CS_INDEX);  
  Bus2IP_RdCE               <= Bus2IP_RdCE_vec(USER00_CE_INDEX);
  Bus2IP_WrCE               <= Bus2IP_WrCE_vec(USER00_CE_INDEX);

-- instantiate the OPB IPIF
  OPB_IPIF_I : opb_ipif
    generic map
    (
      C_ARD_ID_ARRAY              => ARD_ID_ARRAY,
      C_ARD_ADDR_RANGE_ARRAY      => ARD_ADDR_RANGE_ARRAY,
      C_ARD_DWIDTH_ARRAY          => ARD_DWIDTH_ARRAY,
      C_ARD_NUM_CE_ARRAY          => ARD_NUM_CE_ARRAY,
      C_ARD_DEPENDENT_PROPS_ARRAY => ARD_DEPENDENT_PROPS_ARRAY,
      C_PIPELINE_MODEL            => C_PIPELINE_MODEL,
      C_DEV_BLK_ID                => C_USER_ID_CODE,
      C_DEV_MIR_ENABLE            => 1,
      C_AWIDTH                    => C_OPB_AWIDTH,
      C_DWIDTH                    => C_OPB_DWIDTH,
      C_FAMILY                    => C_FAMILY,
      C_IP_INTR_MODE_ARRAY        => IP_INTR_MODE_ARRAY,
      C_INCLUDE_DEV_ISC           => INCLUDE_DEV_ISC,
      C_INCLUDE_DEV_IID           => INCLUDE_DEV_IID,
      C_DEV_BURST_ENABLE          => DEV_BURST_ENABLE
    )
    port map
    (
      OPB_select         => OPB_select,
      OPB_DBus           => OPB_DBus,
      OPB_ABus           => OPB_ABus,
      OPB_BE             => OPB_BE,
      OPB_RNW            => OPB_RNW,
      OPB_seqAddr        => OPB_seqAddr,
      OPB_xferAck        => '0',
      Sln_DBus           => Sln_DBus,
      Sln_xferAck        => Sln_xferAck,
      Sln_errAck         => Sln_errAck,
      Sln_retry          => Sln_retry,
      Sln_toutSup        => Sln_toutSup,
      Bus2IP_CS          => Bus2IP_CS_vec,
      Bus2IP_CE          => open,
      Bus2IP_RdCE        => Bus2IP_RdCE_vec,
      Bus2IP_WrCE        => Bus2IP_WrCE_vec,
      Bus2IP_Data        => Bus2IP_Data,
      Bus2IP_Addr        => Bus2IP_Addr,
      Bus2IP_BE          => Bus2IP_BE,
      Bus2IP_RNW         => Bus2IP_RNW,
      Bus2IP_Burst       => Bus2IP_Burst,
      IP2Bus_Data        => IP2Bus_Data_vec,
      IP2Bus_Ack         => IP2Bus_Ack_vec,
      IP2Bus_Error       => IP2Bus_Error_vec,
      IP2Bus_Retry       => IP2Bus_Retry_vec,
      IP2Bus_ToutSup     => IP2Bus_ToutSup_vec,
      IP2Bus_PostedWrInh => IP2Bus_PostedWrInh_vec,
      OPB_Clk            => OPB_Clk,
      Bus2IP_Clk         => Bus2IP_Clk,
      IP2Bus_Clk         => '0',
      Reset              => OPB_Rst,
      Bus2IP_Reset       => Bus2IP_Reset,
      IP2Bus_Intr        => IP2Bus_Intr,
      Device_Intr        => IP2INTC_Irpt
    );

end architecture imp;

