-- ==============================================================
-- FILE : BRAM2USER.VHD
-- ==============================================================
--
-- This file is part of the FFT example for the Power PC.
--
-- This file is derived from Xilinx provided example code.
--
-- ==============================================================
--
--  Date        : 11/11/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : BRAM to USER interface
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    11-11-04    First Written
--
-- ==============================================================


library ieee;
  use ieee.std_logic_1164.all;

--  Add user library if this source file uses
--  entities from the user library
library bram2user; --USER-- library name
use bram2user.all; --USER-- use statement

-------------------------------------------------------------------------------
-- Port Declarations
-------------------------------------------------------------------------------
-- Definition of Generics:
--      C_MEMSIZE                  -- Memory Size
--      C_PORT_DWIDTH              -- Data bus width
--      C_PORT_AWIDTH              -- Address bus width
--      C_NUM_WE                   -- Number of write enables
--      C_FAMILY                   -- Target FPGA architecture
--
-- Definition of Ports:
--  BRAM Port B
--      BRAM_Rst                   -- BRAM Port B reset
--      BRAM_Clk                   -- BRAM Port B clock
--      BRAM_EN                    -- BRAM Port B enable
--      BRAM_WEN                   -- BRAM Port B write enable
--      BRAM_Addr                  -- BRAM Port B address
--      BRAM_Din                   -- BRAM Port B data in
--      BRAM_Dout                  -- BRAM Port B data out
-- User Logic
--      ip_bram_rst                -- BRAM Port B reset
--      ip_bram_clk                -- BRAM Port B clock
--      ip_bram_en                 -- BRAM Port B enable
--      ip_bram_wen                -- BRAM Port B write enable
--      ip_bram_addr               -- BRAM Port B address
--      ip_bram_din                -- BRAM Port B data in
--      ip_bram_dout               -- BRAM Port B data out
-------------------------------------------------------------------------------
entity bram2user is  --USER-- change entity name
  generic
  (
    C_MEMSIZE     : integer := 2048;
    C_PORT_DWIDTH : integer := 32;
    C_PORT_AWIDTH : integer := 32;
    C_NUM_WE      : integer := 4;
    C_FAMILY      : string  := "virtex2"
  );
  port
  (
    BRAM_Rst_B   : out std_logic;
    BRAM_Clk_B   : out std_logic;
    BRAM_EN_B    : out std_logic;
    BRAM_WEN_B   : out std_logic_vector(0 to C_NUM_WE-1);
    BRAM_Addr_B  : out std_logic_vector(0 to C_PORT_AWIDTH-1);
    BRAM_Din_B   : in  std_logic_vector(0 to C_PORT_DWIDTH-1);
    BRAM_Dout_B  : out std_logic_vector(0 to C_PORT_DWIDTH-1);

    ip_bram_rst  : in  std_logic;
    ip_bram_clk  : in  std_logic;
    ip_bram_en   : in  std_logic;
    ip_bram_wen  : in  std_logic_vector(0 to C_NUM_WE-1);
    ip_bram_addr : in  std_logic_vector(0 to C_PORT_AWIDTH-1);
    ip_bram_din  : in  std_logic_vector(0 to C_PORT_DWIDTH-1);
    ip_bram_dout : out std_logic_vector(0 to C_PORT_DWIDTH-1)
  );
end entity bram2user; --USER-- change entity name

-------------------------------------------------------------------------------
-- Architecture Section
-------------------------------------------------------------------------------

architecture imp of bram2user is --USER-- change entity name

------------------------------------------------------------------------------
begin
------------------------------------------------------------------------------

  BRAM_Rst_B  <= ip_bram_rst;
  BRAM_Clk_B  <= ip_bram_clk;
  BRAM_EN_B   <= ip_bram_en;
  BRAM_WEN_B  <= ip_bram_wen;
  BRAM_Addr_B <= ip_bram_addr;
  BRAM_Dout_B <= ip_bram_din;

  ip_bram_dout <= BRAM_Din_B;

end architecture imp;

